package net.greenfieldtech.cloudonixsdk.cloudonixdialer.fragments;

import android.nfc.Tag;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.support.annotation.Nullable;
import android.support.design.widget.Snackbar;
import android.text.TextUtils;
import android.util.Log;
import android.util.TypedValue;
import android.view.Gravity;
import android.view.View;
import android.widget.RelativeLayout;
import android.widget.TextView;
import butterknife.BindView;
import butterknife.OnClick;
import butterknife.OnLongClick;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.R;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.events.UpdateSettingsEvent;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.models.RegistrationFreeCall;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.SIPConfig;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.SdkFacade;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.Utils;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.VoIPClient;
import net.greenfieldtech.cloudonixsdk.utils.SDKLogger;
import net.greenfieldtech.cloudonixsdk.utils.StorageAndPrefsWrapper;
import net.greenfieldtech.cloudonixsdk.utils.StorageAndPrefsWrapper.PreferencesKeys;
import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;

/**
 * Created by alex on 20.04.16.
 */

public class KeypadFragment extends BaseFragment {

  private static final String TAG = "PhoneDialer";
  private static final int NUMBER_LENGTH_LIMIT = 32;
  private static final int UPDATE_REGISTRATION_INTERVAL = 3000;

  @BindView(R.id.root)
  RelativeLayout root;
  @BindView(R.id.tv_number)
  TextView tvNumber;
  @BindView(R.id.tv_registration_status)
  TextView tvRegStatus;

  private String registrationStatus;
  private String number;
  private boolean mIsFirstTime = true;
  private Handler updateRegistrationHandler;

  private float density;
  private int textSizeStep = 0;

  private String previousNumber;
  private DatabaseReference registrationFreeUser;
  private StorageAndPrefsWrapper prefs = StorageAndPrefsWrapper.getInstance();

  @Override
  public int getLayoutResource() {
    return R.layout.fragment_keypad;
  }

  @Override
  public void onCreate(@Nullable Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);

    density = getResources().getDisplayMetrics().density;

    number = "";
    updateRegistrationHandler = new Handler();
    updateRegistrationHandler.post(updateRegistrationRunnable);
  }

  @Override
  public void onViewCreated(View view, @Nullable Bundle savedInstanceState) {
    super.onViewCreated(view, savedInstanceState);

    VoIPClient.getInstance().setMediaResources(getActivity(), R.raw.busy, R.raw.ringing,
        R.raw.alert, R.raw.end_call, R.raw.hold, R.raw.call_on_hold, R.raw.call_off_hold);

    VoIPClient.getInstance().setLogLevel(SDKLogger.LOG_LEVEL_ALL);
  }

  @Override
  public void onResume() {
    super.onResume();

    tvNumber.setText(number);
    tvRegStatus.setText(registrationStatus);
    updateRegistrationHandler.postDelayed(updateRegistrationRunnable, UPDATE_REGISTRATION_INTERVAL);

    new Handler().postDelayed(new Runnable() {
      @Override
      public void run() {
        if (getActivity() != null) {
          getActivity().runOnUiThread(mAskPermissionsRunnable);
        }
      }
    }, 1000);
  }

  @Override
  public void onPause() {
    super.onPause();

    number = tvNumber.getText().toString();
    updateRegistrationHandler.removeCallbacks(updateRegistrationRunnable);
  }

  private Runnable updateRegistrationRunnable = new Runnable() {
    @Override
    public void run() {
      if (Utils.isRegistrationFree()) {
        String user = prefs.getString(StorageAndPrefsWrapper.PreferencesKeys.ACCOUNT_USERNAME, "");
        registrationStatus = String.format(getString(R.string.keypad_status_registration_free), user);
        tvRegStatus.setText(registrationStatus);
        subscribeRegistrationFree();
      } else {
        updateRegistrationStatus();
        updateRegistrationHandler.postDelayed(this, UPDATE_REGISTRATION_INTERVAL);
      }
    }
  };

  private Runnable mAskPermissionsRunnable = new Runnable() {
    @Override
    public void run() {
      if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
        if (mIsFirstTime) {
          mIsFirstTime = false;
          VoIPClient.getInstance().askForPermissions(getActivity(), true);
        } else {
          VoIPClient.getInstance().askForPermissions(getActivity(), mIsFirstTime);
        }
      }
    }
  };

  private void updateRegistrationStatus() {
    if (isRegistered()) {
      String user = prefs.getString(StorageAndPrefsWrapper.PreferencesKeys.ACCOUNT_USERNAME, "");
      registrationStatus = String.format(getString(R.string.keypad_status_registered), user);
    } else {
      onMessageEvent(null);
      registrationStatus = getString(R.string.keypad_status_not_registered);
    }
    tvRegStatus.setText(registrationStatus);
  }

  private boolean isRegistered() {
    return VoIPClient.getInstance().isRegistered();
  }

  @OnClick(R.id.btn_delete)
  void deleteClicked() {
    if (number.length() > 0) {
      number = number.substring(0, number.length() - 1);
      tvNumber.setText(number);
    }
//
//    float textSize;
//    if (number.length() < 10) {
//      textSize = getResources().getDimension(R.dimen.default_size) / density;
//      textSizeStep--;
//    } else if (number.length() < 16) {
//      textSize = getResources().getDimension(R.dimen.pre_default_text_size) / density;
//      textSizeStep--;
//    } else if (number.length() < 19) {
//      textSize = getResources().getDimension(R.dimen.middle_text_size) / density;
//      textSizeStep--;
//    } else {
//      textSize = getResources().getDimension(R.dimen.min_text_size) / density;
//      textSizeStep--;
//    }
//
//    tvNumber.setTextSize(TypedValue.COMPLEX_UNIT_SP, textSize);
//    tvNumber.setGravity(Gravity.CENTER_VERTICAL | Gravity.CENTER_HORIZONTAL);
    setDefaultTextSize();
  }

  @OnLongClick(R.id.btn_delete)
  boolean deleteNumber() {
    number = "";
    tvNumber.setText(number);
    setDefaultTextSize();
    return true;
  }

  private void setDefaultTextSize() {
    if (TextUtils.isEmpty(tvNumber.getText())) {
      textSizeStep = 0;
      tvNumber.setTextSize(TypedValue.COMPLEX_UNIT_SP,
          getResources().getDimension(R.dimen.default_size) / density);
    }
  }

  @OnClick({R.id.btn_1, R.id.btn_2, R.id.btn_3, R.id.btn_4, R.id.btn_5, R.id.btn_6,
      R.id.btn_7, R.id.btn_8, R.id.btn_9, R.id.btn_0, R.id.btn_star, R.id.btn_pound})
  void onBtnNumberClick(View button) {
    if (tvNumber.getText().length() < NUMBER_LENGTH_LIMIT) {
      try {
        String viewTag = (String) button.getTag();
        try {
          int d = Integer.parseInt(viewTag);
          number = number + d;
          Log.i(TAG, "Number:" + number);
        } catch (NumberFormatException e) {
          if (viewTag.equals("*")) {
            number += "*";
          }
          if (viewTag.equals("#")) {
            number += "#";
          }
        }
        tvNumber.setText(number);
//        customizeTextIfNeed();
      } catch (ClassCastException e) {

      }
    } else {
      Snackbar.make(root, "Limit of number length", Snackbar.LENGTH_SHORT).show();
    }
  }

  private void customizeTextIfNeed() {
    int linesCount = tvNumber.getLineCount();
    float textSize;
    if (linesCount > 1) {
      switch (textSizeStep) {
        case 0: {
          textSize = getResources().getDimension(R.dimen.pre_default_text_size) / density;
          break;
        }
        case 1: {
          textSize = getResources().getDimension(R.dimen.middle_text_size) / density;
          break;
        }
        case 2: {
          textSize = getResources().getDimension(R.dimen.min_text_size) / density;
          break;
        }
        default:
          textSize = getResources().getDimension(R.dimen.min_text_size) / density;
          break;
      }
      tvNumber.setTextSize(TypedValue.COMPLEX_UNIT_SP, textSize);
      textSizeStep++;
      tvNumber.setGravity(Gravity.CENTER_VERTICAL | Gravity.CENTER_HORIZONTAL);
    }
  }

  @OnLongClick(R.id.btn_0)
  boolean addPlusSymbol() {
    number = number + "+";
    tvNumber.setText(number);
    return true;
  }

  @OnClick(R.id.btn_call)
  void doCall() {
    if (!Utils.isRegistrationFree()) {
      updateRegistrationStatus();
    }
    if (!SdkFacade.isCodecSelected()) {
      Snackbar.make(root, getString(R.string.keypad_error_no_codec), Snackbar.LENGTH_SHORT).show();
      return;
    }
    if (TextUtils.isEmpty(number)) {
      if (previousNumber != null) {
        number = previousNumber;
        tvNumber.setText(previousNumber);
      }
    } else {
      if (prefs.getBoolean(PreferencesKeys.USE_REGISTRATION_FREE, false)) {
        VoIPClient.getInstance().dialRegistrationFree(number);
      } else {
        if (isRegistered()) {
          previousNumber = number;
          VoIPClient.getInstance().dial(number);
        } else {
          Snackbar
              .make(root, getString(R.string.keypad_error_not_registered), Snackbar.LENGTH_SHORT)
              .show();
        }
      }
    }
  }

  @Override
  public void onStart() {
    super.onStart();
    EventBus.getDefault().register(this);
  }

  @Override
  public void onStop() {
    EventBus.getDefault().unregister(this);
    super.onStop();
  }

  @Subscribe(sticky = true, threadMode = ThreadMode.MAIN)
  public void onMessageEvent(UpdateSettingsEvent event) {
    // TODO: add to test app configs:
    // ENABLE_NAT
    // ENABLE_STUN
    // DISABLE_SECURE_SIPS
    // STUN_SERVER

    unsubscribeRegistrationFree();

    SIPConfig conf = SIPConfig.getInstance();
    conf.setUserAgent(prefs.getString(StorageAndPrefsWrapper.PreferencesKeys.USER_AGENT, "Greenfield"));
    conf.setUseOpus(prefs.getBoolean(StorageAndPrefsWrapper.PreferencesKeys.USE_OPUS, true));
    conf.setUseILBC(prefs.getBoolean(StorageAndPrefsWrapper.PreferencesKeys.USE_ILBC, true));
    conf.setUseG722(prefs.getBoolean(StorageAndPrefsWrapper.PreferencesKeys.USE_G722, true));
    conf.setUseG711(prefs.getBoolean(StorageAndPrefsWrapper.PreferencesKeys.USE_G711, true));
    conf.setMinAudioRate(prefs.getString(StorageAndPrefsWrapper.PreferencesKeys.MIN_RATE, "8000"));
    conf.setMaxAudioRate(prefs.getString(StorageAndPrefsWrapper.PreferencesKeys.MAX_RATE, "8000"));

    conf.setEnableIce(prefs.getBoolean(StorageAndPrefsWrapper.PreferencesKeys.ENABLE_ICE, true));

    VoIPClient.getInstance().setStoredConfiguration();

    if (SettingsFragment.mRequireSdkReboot) {
      Log.d(TAG, "Restarting VoIPClient ");
      VoIPClient.getInstance().restartSip();
      SettingsFragment.mRequireSdkReboot = false;
    }
    EventBus.getDefault().removeStickyEvent(UpdateSettingsEvent.class);
  }

  // Only for registration free dialing feature
  private void subscribeRegistrationFree() {
    Log.d(TAG,"subscribing to reg-free");
    if (Utils.isRegistrationFree() && registrationFreeUser == null) {
      registrationFreeUser = FirebaseDatabase.getInstance().getReference()
              .child("subscribers")
              .child("incoming")
              .child(prefs.getString(PreferencesKeys.ACCOUNT_USERNAME, ""));
      registrationFreeUser.addValueEventListener(databaseListener);
    }
  }

  // Only for registration free dialing feature
  private void unsubscribeRegistrationFree() {
    if (registrationFreeUser != null) {
      registrationFreeUser.removeEventListener(databaseListener);
      registrationFreeUser = null;
    }
  }

  private void clearFirebaseCall() {
    if (Utils.isRegistrationFree()) {
      DatabaseReference child = FirebaseDatabase.getInstance()
          .getReference()
          .child("subscribers")
          .child("incoming")
          .child(prefs.getString(StorageAndPrefsWrapper.PreferencesKeys.ACCOUNT_USERNAME, ""));
      child.removeValue();
    }
  }

  private ValueEventListener databaseListener = new ValueEventListener() {

    @Override
    public void onDataChange(DataSnapshot dataSnapshot) {

        RegistrationFreeCall data = null;

      if(!dataSnapshot.hasChildren())
          return;

      for (DataSnapshot snapshot : dataSnapshot.getChildren()) {
        data = snapshot.getValue(RegistrationFreeCall.class);
      }
     if (data != null && !TextUtils.isEmpty(data.getCaller()) && !TextUtils.isEmpty(data.getSession())) {
        Log.d(TAG, "Notifying reg-free");
        VoIPClient.getInstance().notifyRegistrationFreeIncoming(data.getCaller(), data.getSession());
      }

      if(data==null)
        Log.d(TAG,"data is null");
      if(data !=null && TextUtils.isEmpty(data.getCaller()))
        Log.d(TAG, "No caller");
      if(data!=null && TextUtils.isEmpty(data.getSession()))
        Log.d(TAG,"No session");

      clearFirebaseCall();
    }

    @Override
    public void onCancelled(DatabaseError error) {
      Log.d(TAG, "Failed to read app title value.", error.toException());
    }

  };

}
