package net.greenfieldtech.cloudonixsdk.cloudonixdialer.fragments;

import android.annotation.TargetApi;
import android.media.Ringtone;
import android.media.RingtoneManager;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.preference.ListPreference;
import android.preference.Preference;
import android.preference.PreferenceCategory;
import android.preference.PreferenceFragment;
import android.preference.RingtonePreference;
import android.preference.SwitchPreference;
import android.text.TextUtils;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.R;
import net.greenfieldtech.cloudonixsdk.utils.StorageAndPrefsWrapper;
import net.greenfieldtech.cloudonixsdk.utils.StorageAndPrefsWrapper.PreferencesKeys;

/**
 * Created by alex on 20.05.16.
 */

@TargetApi(Build.VERSION_CODES.HONEYCOMB)
public class SettingsFragment extends PreferenceFragment {

  private static StorageAndPrefsWrapper prefs;

  @Override
  public void onCreate(final Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);

    prefs = StorageAndPrefsWrapper.getInstance();

    setupSimplePreferencesScreen();
  }

  private void setupSimplePreferencesScreen() {
    addPreferencesFromResource(R.xml.pref_general);

    PreferenceCategory transportsHeader = new PreferenceCategory(getActivity());
    transportsHeader.setTitle(R.string.pref_header_transports);
    getPreferenceScreen().addPreference(transportsHeader);
    addPreferencesFromResource(R.xml.pref_transport_types);

    PreferenceCategory codecsHeader = new PreferenceCategory(getActivity());
    codecsHeader.setTitle(R.string.pref_header_codecs);
    getPreferenceScreen().addPreference(codecsHeader);
    addPreferencesFromResource(R.xml.pref_codecs_settings);

    bindPreferenceSummaryToValue(findPreference(PreferencesKeys.ACCOUNT_SERVER));
    bindPreferenceSummaryToValue(findPreference(PreferencesKeys.ACCOUNT_USERNAME));
    bindPreferenceSummaryToValue(findPreference("account_domain"));
    bindPreferenceSummaryToValue(findPreference(PreferencesKeys.ACCOUNT_DISPLAYNAME));
    bindPreferenceSummaryToValue(findPreference(PreferencesKeys.ACCOUNT_PASSWORD));
    bindPreferenceSummaryToValue(findPreference(PreferencesKeys.ACCOUNT_TRANSPORT));
    bindPreferenceSummaryToValue(findPreference(PreferencesKeys.ACCOUNT_TRANSPORT_PORT));
    bindPreferenceSummaryToValue(findPreference(PreferencesKeys.ACCOUNT_USE_DNS_SRV));
    bindPreferenceSummaryToValue(findPreference(PreferencesKeys.ENABLE_ICE));
    bindPreferenceSummaryToValue(findPreference(PreferencesKeys.USE_REGISTRATION_FREE));
    bindPreferenceSummaryToValue(findPreference(PreferencesKeys.MIN_RATE));
    bindPreferenceSummaryToValue(findPreference(PreferencesKeys.MAX_RATE));
    bindPreferenceSummaryToValue(findPreference(PreferencesKeys.USE_OPUS));
    bindPreferenceSummaryToValue(findPreference(PreferencesKeys.USE_ILBC));
    bindPreferenceSummaryToValue(findPreference(PreferencesKeys.USE_G722));
    bindPreferenceSummaryToValue(findPreference(PreferencesKeys.USE_G711));
  }

  public static boolean mRequireSdkReboot = false;

  private static Preference.OnPreferenceChangeListener sBindPreferenceSummaryToValueListener =
      new Preference.OnPreferenceChangeListener() {

        @Override
        public boolean onPreferenceChange(Preference preference, Object value) {
          String stringValue = value.toString();
          String key = preference.getKey();

          switch (key) {
            case StorageAndPrefsWrapper.PreferencesKeys.ENABLE_ICE:
            case StorageAndPrefsWrapper.PreferencesKeys.USE_OPUS:
            case StorageAndPrefsWrapper.PreferencesKeys.USE_ILBC:
            case StorageAndPrefsWrapper.PreferencesKeys.USE_G722:
            case StorageAndPrefsWrapper.PreferencesKeys.USE_G711:
            case StorageAndPrefsWrapper.PreferencesKeys.USER_AGENT:
              mRequireSdkReboot = true;
              break;
          }

          if (preference instanceof ListPreference) {
            ListPreference listPreference = (ListPreference) preference;
            int index = listPreference.findIndexOfValue(stringValue);

            preference.setSummary(index >= 0 ? listPreference.getEntries()[index] : null);

          } else if (preference instanceof RingtonePreference) {
            if (TextUtils.isEmpty(stringValue)) {
              preference.setSummary(R.string.pref_ringtone_silent);
            } else {
              Ringtone ringtone = RingtoneManager
                  .getRingtone(preference.getContext(), Uri.parse(stringValue));

              if (ringtone == null) {
                preference.setSummary(null);
              } else {
                String name = ringtone.getTitle(preference.getContext());
                preference.setSummary(name);
              }
            }
          } else {
            preference.setSummary(stringValue);
          }
          return true;
        }
      };

  private static void bindPreferenceSummaryToValue(Preference preference) {
    preference.setOnPreferenceChangeListener(sBindPreferenceSummaryToValueListener);

    if (preference instanceof SwitchPreference) {
      sBindPreferenceSummaryToValueListener
          .onPreferenceChange(preference, prefs.getBoolean(preference.getKey(), true));
      return;
    }
    sBindPreferenceSummaryToValueListener
        .onPreferenceChange(preference, prefs.getString(preference.getKey(), ""));
  }

  @TargetApi(Build.VERSION_CODES.HONEYCOMB)
  public static class GeneralPreferenceFragment extends PreferenceFragment {

    @Override
    public void onCreate(Bundle savedInstanceState) {
      super.onCreate(savedInstanceState);
      addPreferencesFromResource(R.xml.pref_general);
    }
  }

  @TargetApi(Build.VERSION_CODES.HONEYCOMB)
  public static class NotificationPreferenceFragment extends PreferenceFragment {

    @Override
    public void onCreate(Bundle savedInstanceState) {
      super.onCreate(savedInstanceState);
      bindPreferenceSummaryToValue(findPreference("notifications_new_message_ringtone"));
    }

  }

  @TargetApi(Build.VERSION_CODES.HONEYCOMB)
  public static class DataSyncPreferenceFragment extends PreferenceFragment {

    @Override
    public void onCreate(Bundle savedInstanceState) {
      super.onCreate(savedInstanceState);
      addPreferencesFromResource(R.xml.pref_transport_types);
      bindPreferenceSummaryToValue(findPreference("sync_frequency"));
    }

  }

  @TargetApi(Build.VERSION_CODES.HONEYCOMB)
  public static class CodecsSyncPreferenceFragment extends PreferenceFragment {

    @Override
    public void onCreate(Bundle savedInstanceState) {
      super.onCreate(savedInstanceState);
      addPreferencesFromResource(R.xml.pref_codecs_settings);
      bindPreferenceSummaryToValue(findPreference("sync_frequency"));
    }

  }

}
