package net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.support.v4.app.ActivityCompat;
import android.text.TextUtils;
import android.widget.Toast;
import android.util.Log;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import net.greenfieldtech.cloudonixsdk.api.interfaces.IVoIPObserver;
import net.greenfieldtech.cloudonixsdk.api.models.RegistrationData;
import net.greenfieldtech.cloudonixsdk.api.models.SDKConstants;
import net.greenfieldtech.cloudonixsdk.api.models.StatisticsData;
import net.greenfieldtech.cloudonixsdk.api.rest.RestClient;
import net.greenfieldtech.cloudonixsdk.api.rest.RestClient.RegistrationFreeSessionCallback;
import net.greenfieldtech.cloudonixsdk.api.rest.registration_free_dialing.RegistrationFreeDialingResponse;
import net.greenfieldtech.cloudonixsdk.appinterface.CloudonixSDKClient;
import net.greenfieldtech.cloudonixsdk.appinterface.SIPHeader;
import net.greenfieldtech.cloudonixsdk.appinterface.VoipCall;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.R;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.activities.CallActivity;
import net.greenfieldtech.cloudonixsdk.utils.CryptUtils;
import net.greenfieldtech.cloudonixsdk.utils.SDKLogger;
import net.greenfieldtech.cloudonixsdk.utils.StorageAndPrefsWrapper;
import net.greenfieldtech.cloudonixsdk.utils.StorageAndPrefsWrapper.PreferencesKeys;

public class VoIPClient implements IVoIPObserver {

  private static final String TAG = "VoIPClient";

  public static String INTENT_CALLING_ACTION = "net.greenfieldtech.cloudonixsdk.CALL_ACTIVE";
  public static String INTENT_RINGING_ACTION = "net.greenfieldtech.cloudonixsdk.CALL_RINGING";
  public static String INTENT_EARLY_MEDIA_ACTION = "net.greenfieldtech.cloudonixsdk.CALL_EARLY_MEDIA";
  public static String INTENT_CALL_HANGUP = "net.greenfieldtech.cloudonixsdk.CALL_HANGUP";
  public static String INTENT_INCOMING_CALL = "net.greenfieldtech.cloudonixsdk.CALL_INCOMING";

  private static VoIPClient mInstance;
  private CloudonixSDKClient sdkClient;
  private static String LICENSE_KEY;

  private boolean isSpeakerOn = false;

  public static VoIPClient getInstance() {
    return mInstance;
  }

  private CloudonixSDKClient getWrapper() {
    if (sdkClient == null) {
      sdkClient = CloudonixSDKClient.getInstance(LICENSE_KEY);
    }
    return sdkClient;
  }

  private VoIPClient(Context context) {
    getWrapper().setLogLevel(SDKLogger.LOG_LEVEL_ALL);
    getWrapper().bind(context);

    getWrapper().initPreferences(context);
    SIPConfig.create(context);

    getWrapper().addEventsListener(this);
    getWrapper().setAllowIncomingCallIntent(INTENT_INCOMING_CALL);

    Intent intent = new Intent(context, CallActivity.class);
    intent.setAction("net.greenfieldtech.sipsdk.ACTIVE_CALL");
    getWrapper().setNotificationResources(intent, R.mipmap.ic_launcher,
        context.getString(R.string.app_name), "SDK is running and has an active call");
    getWrapper().setForegroundInitRes(R.mipmap.ic_launcher,
        "VoIP Service","Initializing","VoIP Service initializing");
  }

  public void destroy() {
    getWrapper().removeFromForeground();
    getWrapper().removeEventsListener(this);
    getWrapper().destroy();
    mInstance = null;
  }

  public void shutdown() {
    getWrapper().shutdown();
  }

  public void restartSip() {
    getWrapper().restart();
  }

  public void setConfig(ConfigurationKey key, String value) {
    getWrapper().setConfig(key, value);
  }

  public void commitConfig() {
    getWrapper().commitConfig();
  }

  public StatisticsData getStatistics(String key) {
    return getWrapper().getStatistics(key);
  }

  // First init must be with internet connection !!!
  public static void initWithContext(Context context) {
    LICENSE_KEY = loadLicenseKey(context);
    mInstance = new VoIPClient(context);
  }

  private static String loadLicenseKey(Context context) {
    InputStream input = context.getResources().openRawResource(R.raw.cloudonix_license_key);
    try {
      byte[] inputArray = CryptUtils.convertStreamToByteArray(input);
      return new String(inputArray);
    } catch (IOException e) {
      e.printStackTrace();
    }
    return "";
  }

  public void addCallbacksListener(IVoIPObserver callbacksListener) {
    getWrapper().addEventsListener(callbacksListener);
  }

  public void removeCallbacksListener(IVoIPObserver listener) {
    getWrapper().removeEventsListener(listener);
  }

  public void setMediaResources(Context context,
      int busyTone,
      int ringingTone,
      int alertTone,
      int endCallTone,
      int holdTone,
      int beginHoldTone,
      int endHoldTone) {
    getWrapper().setMediaResources(context, busyTone, ringingTone, alertTone, endCallTone, holdTone,
        beginHoldTone, endHoldTone);
  }

  public boolean isRegistered() {
    return getWrapper().isRegistered();
  }

  public boolean isSpeakerOn() {
    return isSpeakerOn;
  }

  public void setSpeaker(boolean speakerEnabled) {
    isSpeakerOn = speakerEnabled;
    if (isSpeakerOn) {
      getWrapper().setAudioRoute(SDKConstants.AudioRoute.SPEAKER);
    } else {
      getWrapper().setAudioRoute(SDKConstants.AudioRoute.EARPIECE);
    }
  }

  public String getAudioRoute() {
    return getWrapper().getAudioRoute().toString();
  }

  public boolean isBluetoothOn() {
    return getWrapper().isBluetoothEnabled();
  }

  public void setBluetooth(boolean bluetoothEnabled) {
    if (bluetoothEnabled) {
      getWrapper().setAudioRoute(SDKConstants.AudioRoute.BLUETOOTH);
    } else {
      getWrapper().setAudioRoute(SDKConstants.AudioRoute.EARPIECE);
    }
  }

  public void setMute(boolean shouldMute) {
    getWrapper().mute(shouldMute);
  }

  public void setMuteByKey(String key, boolean shouldMute) {
    getWrapper().muteByKey(key, shouldMute);
  }

  public void sendDTMF(String key, char c) {
    getWrapper().DTMF(key, c);
  }

  public boolean isStarted() {
    return getWrapper().isInitialized();
  }

  public void answer(String callKey) {
    getWrapper().answer(callKey);
  }

  public void reject(String callKey) {
    getWrapper().reject(callKey);
  }

  public boolean dial(String number) {
    SDKLogger.d(TAG, "VoIPClient -> dial: " + number);
    if (!getWrapper().isRegistered()) {
      return false;
    }
    getWrapper().dial(number);
    return false;
  }

  public boolean dialWithHeaders(String number, List<SIPHeader> headers) {
    SDKLogger.d(TAG, "VoIPClient -> dial: ");
    if (!getWrapper().isRegistered()) {
      return false;
    }
    getWrapper().dialWithHeaders(number, headers);
    return false;
  }

  public void dialRegistrationFree(final String number) {
    SDKLogger.d(TAG, "VoIPClient -> dial free: " + number);

    StorageAndPrefsWrapper prefs = StorageAndPrefsWrapper.getInstance();
    final String userName = prefs.getString(PreferencesKeys.ACCOUNT_USERNAME, "");
    final String userPass = prefs.getString(PreferencesKeys.ACCOUNT_PASSWORD, "");
    final String domain = prefs.getString("account_domain", "");

    RestClient.createRegistrationFreeSession(number, userName, userPass, domain,
        new RegistrationFreeSessionCallback() {
          @Override
          public void onFailure(IOException e) {
            e.printStackTrace();
          }

          @Override
          public void onSuccess(RegistrationFreeDialingResponse dialingResponse) {
            dialRegistrationFree(number, dialingResponse.getToken());
          }
        });
  }

  public void dialRegistrationFree(final String number, String sessionId) {
    getWrapper().dialRegistrationFree(number, sessionId);
  }

  public void notifyRegistrationFreeIncoming(String callerId, String sessionId) {
    Log.d(TAG,"Caller id:"+callerId+" , and session id is: "+sessionId);
    getWrapper().notifyRegistrationFreeIncoming(callerId, sessionId);
  }

  public void setControlAudioRoute(boolean controlAudioRoute) {
    getWrapper().setControlAudioRoute(controlAudioRoute);
  }

  public boolean isControlAudioRoute() {
    return getWrapper().isControlAudioRoute();
  }

  public void unregister() {
    SDKLogger.d(TAG, "VoIPClient -> unregister()");
    getWrapper().unregister();
  }

  public float getCallMOS(String key) {
    StatisticsData data = getWrapper().getStatistics(key);
    if (data != null) {
      return data.MOS;
    } else {
      return 0;
    }
  }

  public void hangup(String callKey) {
    getWrapper().hangup(callKey);
  }

  public void localHold(String callKey, boolean enabled) {
    getWrapper().localHold(callKey, enabled);
  }

  public void dumpStatistics(boolean b) {
    getWrapper().dumpStatisticsToLog(b);
  }

  public void setLogLevel(int i) {
    getWrapper().setLogLevel(SDKLogger.LOG_LEVEL_ALL);
  }

  public void setEcFixedDelay(int fixedEcDelay) {
    getWrapper().setEcFixedDelay(fixedEcDelay);
  }

  public CallState getCallState(String callKey) {
    return getWrapper().getCallState(callKey);
  }

  public String getCallUrl(String key) {
    return getWrapper().getCallUrl(key);
  }

  public int getNumberOfCalls() {
    return getWrapper().getNumberOfCalls();
  }

  public String getCallKeyForIndex(int index) {
    return getWrapper().getCallKeyForIndex(index);
  }

  public VoipCall[] getCalls() {
    return getWrapper().getCalls();
  }

  public void removeForegroundService() {
    getWrapper().removeFromForeground();
  }

  public void addForegroundService() {
    // Call this only after setting notification resources via:
    // setNotificationResources()
    getWrapper().addToForeground();
  }

  public void askForPermissions(Activity activity, boolean isFirstTime) {
    String[] MANDATORY_PERMISSIONS = {Manifest.permission.RECORD_AUDIO,
        Manifest.permission.BROADCAST_STICKY, Manifest.permission.RECEIVE_BOOT_COMPLETED,
        Manifest.permission.INTERNET,
        Manifest.permission.ACCESS_WIFI_STATE, Manifest.permission.ACCESS_NETWORK_STATE,
        Manifest.permission.PROCESS_OUTGOING_CALLS, Manifest.permission.MODIFY_AUDIO_SETTINGS,
        Manifest.permission.BLUETOOTH,
        Manifest.permission.BLUETOOTH_ADMIN, Manifest.permission.WAKE_LOCK,
        Manifest.permission.DISABLE_KEYGUARD, Manifest.permission.VIBRATE,
        Manifest.permission.WRITE_EXTERNAL_STORAGE,
        Manifest.permission.READ_PHONE_STATE, Manifest.permission.CHANGE_WIFI_STATE};

    if (isFirstTime) {
      ActivityCompat.requestPermissions(activity, MANDATORY_PERMISSIONS, 100);
    } else {
      for (String permission : MANDATORY_PERMISSIONS) {
        if (activity.checkCallingOrSelfPermission(permission)
            != PackageManager.PERMISSION_GRANTED) {
          Toast.makeText(activity, "Permission " + permission
                  + " is not granted. Please accept it for the app to work correctly.",
              Toast.LENGTH_SHORT).show();
          ActivityCompat.requestPermissions(activity, new String[]{permission}, 100);
          return;
        }
      }
    }
  }

  @Override
  public void onSipStarted() {
    SDKLogger.d(TAG, "VoIPClient -> onSipStarted");
    if (!Utils.isRegistrationFree()) {
      VoIPClient.getInstance().registerAccount();
    }
  }

  @Override
  public void onSipStartFailed() {
    SDKLogger.d(TAG, "VoIPClient -> onSipStartFailed: account error unrecoverable.");
  }

  @Override
  public void onRegisterState(RegisterState result, int expiry) {
    switch (result) {
      case REGISTRATION_SUCCESS:
        SDKLogger.d(TAG, "VoIPClient -> Account is now registered. Registery will expire in: "
            + expiry + " ms.");
        break;
      case REGISTRATION_ERROR_CREDENTIALS:
        SDKLogger.d(TAG, "VoIPClient -> onError: account error unrecoverable.");
        break;
      case REGISTRATION_UNREGISTERED:
        SDKLogger.d(TAG, "VoIPClient -> No account is registered!");
        if (Utils.isRegistrationFree()) {
          SDKLogger.d(TAG, "VoIPClient -> Set configurations for registration-free calls.");
          getWrapper().setConfiguration(getStoredConfiguration());
        }
        break;
      default:
        break;
    }
  }

  @Override
  public void onCallState(String key, CallState callState, String contactUrl) {
    switch (callState) {
      case CALL_STATE_STARTING:
        SDKLogger.d(TAG, "VoIPClient -> onStarting: " + key + " to number: " + contactUrl);
        break;
      case CALL_STATE_CONNECTING:
        SDKLogger.d(TAG, "VoIPClient -> onConnecting: " + key + " to number: " + contactUrl);
        break;
      case CALL_STATE_CALLING:
        SDKLogger.d(TAG, "VoIPClient -> onCalling: " + key + " to number: " + contactUrl);
        break;
      case CALL_STATE_RINGING:
        SDKLogger.d(TAG, "VoIPClient -> onRinging: " + key + " Number: " + contactUrl);
        break;
      case CALL_STATE_EARLY:
        SDKLogger.d(TAG, "VoIPClient -> onEarlyMedia: " + key + " Number: " + contactUrl);
        break;
      case CALL_STATE_MEDIAACTIVE:
        SDKLogger.d(TAG, "VoIPClient -> onMediaActive: " + key + " Number: " + contactUrl);
        break;
      case CALL_STATE_CONFIRMED:
        SDKLogger.d(TAG, "VoIPClient -> onConfirmed: " + key + " Number: " + contactUrl);
        break;
      case CALL_STATE_DISCONNECTEDDUETOBUSY:
      case CALL_STATE_DISCONNECTEDMEDIACHANGED:
      case CALL_STATE_DISCONNECTED:
        isSpeakerOn = false;
        SDKLogger.d(TAG, "VoIPClient -> onHangup: " + key + " Number: " + contactUrl);
        break;
      default:
        break;
    }
  }

  @Override
  public void onLog(int level, String message) {

  }

  @Override
  public void onSipStopped() {
    SDKLogger.d(TAG, "VoIPClient -> onSipStopped");
  }

  @Override
  public void onBluetoothState(BluetoothState bluetoothState) {
    SDKLogger.d(TAG, "VoIPClient -> onBluetoothState " + bluetoothState.toString());
  }

  @Override
  public void onAudioRouteChange(AudioRoute state) {
    SDKLogger.d(TAG, "VoIPClient -> onAudioRouteChange " + state.toString());
  }

  @Override
  public void onBluetoothBtnState(BluetoothButtonsState state, String command, String eventName,
      Integer buttonID, Integer mode) {
    SDKLogger.d(TAG, "VoIPClient -> onBluetoothBtnState " + state.toString());
  }

  @Override
  public void onNetworkLost() {
    SDKLogger.d(TAG, "VoIPClient -> onNetworkLost ");
  }

  @Override
  public void onNetworkChanged() {
    SDKLogger.d(TAG, "VoIPClient -> onNetworkChanged ");
  }

  @Override
  public void onNetworkRegained() {
    SDKLogger.d(TAG, "VoIPClient -> onNetworkRegained ");
  }

  @Override
  public void onLicense(LicensingState state, String description) {
    SDKLogger.d(TAG, "VoIPClient -> onLicense ");
    if (state == LicensingState.LICENSING_SUCCESS) {
      setConfigs();
      setStoredConfiguration();
    } else {
      SDKLogger.d(TAG, "License status - " + state + ". Cause - " + description);
    }
  }

  private void setConfigs() {
    SIPConfig config = SIPConfig.getInstance();
    setConfig(ConfigurationKey.USER_AGENT, config.getUserAgent());
    setConfig(ConfigurationKey.USE_OPUS, config.getUseOpus() ? "1" : "0");
    setConfig(ConfigurationKey.USE_G711, config.getUseG711() ? "1" : "0");
    setConfig(ConfigurationKey.USE_G722, config.getUseG722() ? "1" : "0");
    setConfig(ConfigurationKey.USE_ILBC, config.getUseILBC() ? "1" : "0");
    setConfig(ConfigurationKey.MIN_RATE, config.getMinAudioRate());
    setConfig(ConfigurationKey.MAX_RATE, config.getMaxAudioRate());

    setConfig(ConfigurationKey.ENABLE_ICE, config.getEnableIce() ? "1" : "0");
    setConfig(ConfigurationKey.ENABLE_NAT, "1");
    setConfig(ConfigurationKey.DISABLE_SECURE_SIPS, "1");
    commitConfig();
  }

  public void setStoredConfiguration() {
    setConfiguration(getStoredConfiguration());
  }

  public void setConfiguration(RegistrationData registrationData) {
    if (Utils.isRegistrationFree()) {
      getWrapper().unregister();
    }
    getWrapper().setConfiguration(registrationData);
  }

  private RegistrationData getStoredConfiguration() {
    StorageAndPrefsWrapper prefs = StorageAndPrefsWrapper.getInstance();

    RegistrationData registrationData = new RegistrationData();
    registrationData.setServerUrl(prefs.getString(StorageAndPrefsWrapper.PreferencesKeys.ACCOUNT_SERVER, ""));
    registrationData.setUsername(prefs.getString(StorageAndPrefsWrapper.PreferencesKeys.ACCOUNT_USERNAME, ""));
    registrationData.setDisplayName(prefs.getString(StorageAndPrefsWrapper.PreferencesKeys.ACCOUNT_DISPLAYNAME, ""));
    registrationData.setPassword(prefs.getString(StorageAndPrefsWrapper.PreferencesKeys.ACCOUNT_PASSWORD, ""));
    String portStr = prefs.getString(StorageAndPrefsWrapper.PreferencesKeys.ACCOUNT_TRANSPORT_PORT);
    int port = 0;
    if (!TextUtils.isEmpty(portStr)) {
      port = Integer.parseInt(portStr);
    }
    registrationData.setPort(port);
    registrationData.setUseDnsSrv(prefs.getBoolean(StorageAndPrefsWrapper.PreferencesKeys.ACCOUNT_USE_DNS_SRV, false));
    registrationData.setDomain(prefs.getString("account_domain", ""));

    String transportType = prefs.getString(StorageAndPrefsWrapper.PreferencesKeys.ACCOUNT_TRANSPORT, "");
    registrationData.setTransportType(RegistrationData.TransportType.valueOf(transportType));

//    String workflowType = prefs.getString(StorageAndPrefsWrapper.PreferencesKeys.ACCOUNT_WORKFLOW, "");
//    registrationData.setWorkflowType(RegistrationData.WorkflowType.valueOf(workflowType));
    registrationData.setWorkflowType(RegistrationData.WorkflowType.WORKFLOW_TYPE_AUTODETECT);

    return registrationData;
  }

  public void registerAccount() {
    getWrapper().registerAccount();
  }

  @Override
  public void onNATTypeDetected(NATType type) {
    SDKLogger.d(TAG, "VoIPClient -> onNATTypeDetected " + type);
  }

}
